unit u_main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, IBConnection, Forms, Controls, Graphics, Dialogs, StdCtrls,
  RTTIGrids;


type

  { TGridObject }

  TGridObject = class(TPersistent)
  constructor Create();
  private
    FDataBaseHost: String;
    FDatabaseParams: String;
    FDatabasePassword: String;
    FDataBasePath: String;
    FDatabaseUsername: String;
    FFieldCurrency: String;
    FFieldDecimal: String;
    FFieldDouble: String;
    FFieldID: String;
    FInputValue: Currency;
    FTableName: String;
  published
    property DataBaseHost : String read FDataBaseHost write FDataBaseHost;
    property DataBasePath : String read FDataBasePath write FDataBasePath;
    property DatabaseUsername : String read FDatabaseUsername write FDatabaseUsername;
    property DatabasePassword : String read FDatabasePassword write FDatabasePassword;
    property DatabaseParams : String read FDatabaseParams write FDatabaseParams;
    property TableName : String read FTableName write FTableName;
    property FieldID : String read FFieldID write FFieldID;
    property FieldCurrency : String read FFieldCurrency write FFieldCurrency;
    property FieldDecimal : String read FFieldDecimal write FFieldDecimal;
    property FieldDouble : String  read FFieldDouble write FFieldDouble;
    property InputValue : Currency read FInputValue write FInputValue;

  end;

type

  { TFormMain }

  TFormMain = class(TForm)
    btEsegui: TButton;
    dbconn: TIBConnection;
    propertyGrid: TTIPropertyGrid;
    procedure btEseguiClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure caricaImpostazioni();
    procedure salvaImpostazioni();
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
    function connectToTheDatabase() : Boolean;
    function doTheQuery() : Boolean;
  private

  public

     oggetto : TGridObject;
     configFolder : String;
     dbconnesso : Boolean;

  end;

var
  FormMain: TFormMain;

implementation

{$R *.lfm}

uses sqldb, math, lclintf, strutils, dateutils, lcltype, inifiles, FileUtil;

{ TFormMain }

procedure TFormMain.FormCreate(Sender: TObject);
begin
  dbconnesso := False;
  configFolder := GetAppConfigDir(False);
  if DirectoryExists(configFolder) = False then
    ForceDirectories(configFolder);

  oggetto := TGridObject.Create;
  caricaImpostazioni();
  propertyGrid.TIObject := oggetto;
end;

function TFormMain.connectToTheDatabase(): Boolean;
begin

  Result := True;
  try
      if dbconnesso then
        begin
           try
               dbconn.Close(True);

           except on ex2 : Exception do
               begin
                   // do nothing
               end;
           end;
           dbconnesso := False;
        end;


      dbconn.HostName := oggetto.DataBaseHost;
      dbconn.DatabaseName := oggetto.DataBasePath;
      dbconn.UserName := oggetto.DatabaseUsername;
      dbconn.Password := oggetto.DatabasePassword;
      dbconn.Params.Clear;
      dbconn.Open;

      if assigned(dbconn.Transaction) then
        dbconn.Transaction.Free;

      dbconn.Transaction := TSQLTransaction.Create(dbconn);
      Result := True;


  except on ex : Exception do
      begin
         dbconnesso := False;
         Application.MessageBox(PChar('Unexpected error while trying to connect to the Database:' + LineEnding +
          ex.Message), 'Can''t connect', MB_OK + MB_ICONERROR);
      end;
  end;


end;

function TFormMain.doTheQuery(): Boolean;
var
  q : TSQLQuery;
begin

  Result := True;
  q := TSQLQuery.Create(nil);
  try
      q.Database := dbconn;
      q.SQL.Text := 'UPDATE OR INSERT INTO ' + oggetto.TableName + ' (' + oggetto.FieldID + ', ' + oggetto.FieldCurrency + ', ' +
                    oggetto.FieldDecimal + ', ' + oggetto.FieldDouble + ') ' +
                    'VALUES(1, :par0, :par1, :par2) ' +
                    'MATCHING (' + oggetto.FieldID + ') ';

      q.Params[0].asCurrency := oggetto.InputValue;
      q.Params[1].asCurrency := oggetto.InputValue;
      q.Params[2].AsFloat := oggetto.InputValue;
      q.ExecSql;

      dbconn.Transaction.Commit;

      q.SQL.Text := 'SELECT ' + oggetto.FieldCurrency + ' as c0, ' + oggetto.FieldDecimal + ', ' + oggetto.FieldDouble + ' ' +
                    'FROM ' + oggetto.TableName + ' ' +
                    'WHERE ' + oggetto.FieldID + ' = 1 ';
      q.Open;
      if not(q.EOF) then
        begin
           ShowMessage('Query executed successfully: ' + LineEnding +
                       'DECIMAL ' + CurrToStrF(q.Fields[0].asCurrency, ffGeneral, 4) + LineEnding +
                       'NUMERIC ' + CurrToStrF(q.Fields[1].asCurrency, ffGeneral, 4) + LineEnding +
                       'Double ' + CurrToStrF(q.Fields[2].asCurrency, ffGeneral, 2));

        end;
      q.Close;


        {update or insert into Cows (Name, Number, Location)
  values ('Suzy Creamcheese', 3278823, 'Green Pastures')
  matching (Number)}


  except on ex : Exception do
      begin
          Result := False;
          dbconn.Transaction.Rollback;
          Application.MessageBox(PChar('Unexpected error while doing the query:' + LIneEnding +
            ex.Message), 'Error', MB_OK + MB_ICONERROR);
      end;
  end;
  q.Free;

end;

procedure TFormMain.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  propertyGrid.TIObject := nil;
  oggetto.Free;
end;

procedure TFormMain.btEseguiClick(Sender: TObject);
begin
  if connectToTheDatabase() then
    begin
       if doTheQuery() then
         begin
             // do nothing...
         end;
    end;
end;

procedure TFormMain.caricaImpostazioni();
var
  pIni : TINIFile;
begin

  pIni := TINIFile.Create(configFolder + 'config.ini');

  oggetto.DataBaseHost := pIni.ReadString('opzioni', 'databaseHost', oggetto.DataBaseHost);
  oggetto.DataBasePath := pIni.ReadString('opzioni', 'databasePath', oggetto.DataBasePath);
  oggetto.DatabaseUsername := pIni.ReadString('opzioni', 'databaseUsername', oggetto.DatabaseUsername);
  oggetto.DatabasePassword := pIni.ReadString('opzioni', 'databasePassword', Oggetto.DatabasePassword);
  oggetto.DatabaseParams := pIni.ReadString('opzioni', 'databaseParams', oggetto.DatabaseParams);
  oggetto.InputValue := pIni.ReadFloat('opzioni', 'inputValue', oggetto.InputValue);
  oggetto.TableName := pIni.ReadString('opzioni', 'tableName', oggetto.TableName);
  oggetto.FieldID := pIni.ReadString('opzioni', 'fieldID', oggetto.FieldID);
  oggetto.FieldCurrency := pIni.ReadString('opzioni', 'fieldCurrency', oggetto.FieldCurrency);
  oggetto.FieldDecimal := pIni.ReadString('opzioni', 'fieldDecimal', oggetto.FieldDecimal);
  oggetto.FieldDouble := pIni.ReadString('opzioni', 'fieldDouble', oggetto.FieldDouble);

  pIni.Free;

end;

procedure TFormMain.salvaImpostazioni();
var
  pIni : TINIFile;
begin

  pIni := TINIFile.Create(configFolder + 'config.ini');
  pIni.CacheUpdates := True;
  pIni.WriteString('opzioni', 'databaseHost', oggetto.DataBaseHost);
  pIni.WriteString('opzioni', 'databasePath', oggetto.DatabasePath);
  pIni.WriteString('opzioni', 'databaseUsername', oggetto.DatabaseUsername);
  pIni.WriteString('opzioni', 'databasePassword', oggetto.DatabasePassword);
  pIni.WriteString('opzioni', 'databaseParams', oggetto.DatabaseParams);
  pIni.WriteFloat('opzioni', 'inputValue', oggetto.InputValue);
  pIni.WriteString('opzioni', 'tableName', oggetto.TableName);
  pIni.WriteString('opzioni', 'fieldID', oggetto.FieldID);
  pIni.WriteString('opzioni', 'fieldCurrency', oggetto.FieldCurrency);
  pIni.WriteString('opzioni', 'fieldDecimal', oggetto.FieldDecimal);
  pIni.WriteString('opzioni', 'fieldDouble', oggetto.FieldDouble);

  pIni.UpdateFile;
  pIni.Free;

end;

procedure TFormMain.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  salvaImpostazioni();
end;

{ TGridObject }

constructor TGridObject.Create();
begin
  inherited Create();

  FDataBaseHost := '127.0.0.1';
  FDataBasePath := 'C:\InvoiceFactory\backups\CURRENCYTEST.FDB';
  fDatabaseUsername := 'SYSDBA';
  fDatabasePassword := 'masterkey';
  FDatabaseParams := 'isc_tpb_read_committed' + LineEnding +
                     'isc_tpb_wait' + LineEnding +
                     'isc_tpb_no_rec_version';
  fInputValue := 1115;
  FTableName := 'TABLE_TEST';
  FFieldID := 'TEST_ID';
  FFieldCurrency := 'TEST_TYPE1';
  FFieldDecimal := 'TEST_TYPE2';
  FFieldDouble := 'TEST_TYPE3';
end;

end.

